@@ -1,13 +1,35 @@
 """Module for using the entire SAMPLE method pipeline"""
 import copy
 import functools
-from typing import Callable, Optional, List
+from typing import Callable, Dict, List, Optional, Sequence, Union
 
 import numpy as np
 from sklearn import base
 
-from sample.regression import HingeRegression
+from sample import utils
+from sample import hinge
 from sample.sms import mm
+from sample.utils import dsp as dsp_utils
+
+
+@utils.numpy_out(dtype=float)
+def modal_energy(a: np.ndarray,
+                 d: np.ndarray,
+                 out: Optional[np.ndarray] = None) -> np.ndarray:
+  """Compute the integrals of the modal amplitude envelopes
+
+  Args:
+    a (array): Amplitudes
+    d (array): Decays
+    out (array): Optional. Array to use for storing results
+
+  Returns:
+    array: Energies"""
+  # d * a^2 / 4
+  np.square(a, out=out)
+  np.multiply(out, d, out=out)
+  np.true_divide(out, 4, out=out)
+  return out
 
 
 class SAMPLE(base.RegressorMixin, base.BaseEstimator):
@@ -17,7 +39,7 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
     sinusoidal_model: Sinusoidal model. Default is an instance of
       :class:`sample.sms.mm.ModalModel`
     regressor: Regressor. Default is an instance of
-      :class:`sample.regression.HingeRegression`
+      :class:`sample.hinge.HingeRegression`
     regressor_k (str): Attribute name for the estimated slope
       coefficient of :data:`regressor`
     regressor_q (str): Attribute name for the estimated intercept
@@ -34,7 +56,7 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
   def __init__(
       self,
       sinusoidal_model=mm.ModalModel(),
-      regressor=HingeRegression(),
+      regressor=hinge.HingeRegression(),
       regressor_k: str = "k_",
       regressor_q: str = "q_",
       freq_reduce: Callable[[np.ndarray], float] = np.mean,
@@ -77,7 +99,7 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
       SAMPLE: self"""
     self.set_params(**kwargs)
     tracks = self.sinusoidal_model.fit(x, y).tracks_
-    self.param_matrix_ = np.zeros((3, len(tracks)))
+    self.param_matrix_ = np.empty((3, len(tracks)))
     for i, t in enumerate(tracks):
       notnans = np.logical_not(np.isnan(t["mag"]))
       self.param_matrix_[0, i] = self.freq_reduce(t["freq"][notnans])
@@ -90,9 +112,14 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
       self.param_matrix_[1, i] = \
         -40 * np.log10(np.e) / getattr(self.regressor, self.regressor_k)
       self.param_matrix_[2, i] = \
-        2 * 10**(getattr(self.regressor, self.regressor_q) / 20)
+        2 * dsp_utils.db2a(getattr(self.regressor, self.regressor_q))
+    self.param_matrix_ = self.param_matrix_[:, self._valid_params_]
     return self
 
+  @property
+  def _valid_params_(self) -> Sequence[bool]:
+    return np.isfinite(self.param_matrix_).all(axis=0)
+
   @property
   def freqs_(self) -> np.ndarray:
     """Learned modal frequencies"""
@@ -123,7 +150,7 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
   @property
   def energies_(self) -> np.ndarray:
     """Learned modal energies"""
-    return 4 * self.amps_**2 / self.decays_
+    return modal_energy(self.amps_, self.decays_)
 
   def mode_argsort_(self,
                     order: str = "energies",
@@ -177,7 +204,8 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
               x: np.ndarray,
               n_modes: Optional[int] = None,
               order: str = "energies",
-              reverse: bool = True) -> np.ndarray:
+              reverse: bool = True,
+              **kwargs) -> np.ndarray:
     """Resynthesize audio
 
     Args:
@@ -190,6 +218,7 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
         :data:`"amps"` and :data:`"decays"`
       reverse (bool): Whether the order should be reversed (decreasing).
         Defaults to :data:`True`
+      **kwargs: Keyword arguments for :func:`additive_synth`
 
     Returns:
       array: Array of audio samples"""
@@ -199,10 +228,69 @@ class SAMPLE(base.RegressorMixin, base.BaseEstimator):
       else:
         n_modes = self.max_n_modes
     m_ord = self.mode_argsort_(order=order, reverse=reverse)[:n_modes]
-    row = functools.partial(np.reshape, newshape=(1, -1))
-    col = functools.partial(np.reshape, newshape=(-1, 1))
+    return additive_synth(x, self.freqs_[m_ord], self.decays_[m_ord],
+                          self.amps_[m_ord], **kwargs)
 
-    osc = np.sin(2 * np.pi * col(x) @ row(self.freqs_[m_ord]))
-    dec = np.exp(col(x) @ row(-2 / self.decays_[m_ord]))
-    amp = col(self.amps_[m_ord])
-    return np.squeeze((dec * osc) @ amp)
+
+def _random_phases(n: int,
+                   seed: Optional[int] = None,
+                   rng: Optional[np.random.Generator] = None) -> np.ndarray:
+  """Randomize a number of phase values between 0 and 2*pi
+
+  Args:
+    n (int): Number of phase values to sample
+    rng (np.random.Generator): Random number generator
+    seed (int): Seed for random number generator
+
+  Returns:
+    array: Random phase values"""
+  if rng is None:
+    rng = np.random.default_rng(seed=seed)
+  return rng.uniform(0, 2 * np.pi, n)
+
+
+_phases_funcs: Dict[str, Callable[[int], np.ndarray]] = {
+    "random": _random_phases,
+}
+
+
+def additive_synth(x,
+                   freqs: Sequence[float],
+                   decays: Sequence[float],
+                   amps: Sequence[float],
+                   phases: Optional[Union[Sequence[float], str]] = None,
+                   analytical: bool = False,
+                   **kwargs) -> np.array:
+  """Additively synthesize audio
+
+    Args:
+      x (array): Time axis
+      freqs (array): Modal frequencies
+      decays (array): Modal decays
+      amps (array): Modal amplitudes
+      phases (array): Starting phase for every mode, optional
+      analytical (bool): If :data:`True`, use a complex
+        exponential as an oscillator
+      **kwargs: Keyword arguments for random phase generator
+
+    Returns:
+      array: Array of audio samples"""
+  row = functools.partial(np.reshape, newshape=(1, -1))
+  col = functools.partial(np.reshape, newshape=(-1, 1))
+
+  osc = (2 * np.pi * col(x)) @ row(freqs)
+  if phases is not None:
+    if isinstance(phases, str):
+      try:
+        phases = _phases_funcs[phases](np.size(freqs), **kwargs)
+      except KeyError as e:
+        raise ValueError(
+            f"Unsupported option for phases: '{phases}'. "
+            f"Supported options are: {utils.comma_join_quote(_phases_funcs)}"
+        ) from e
+    np.add(osc, row(phases), out=osc)
+  osc = (dsp_utils.expi if analytical else np.cos)(osc)
+  dec = col(x) @ (-2 / row(decays))
+  np.exp(dec, out=dec)
+  amp = col(amps)
+  return np.squeeze((dec * osc) @ amp)
