@@ -1,8 +1,9 @@
 """Signal processing functions for SMS"""
 import functools
-from typing import Optional, Tuple
+from typing import Optional, Tuple, Union
 
 import numpy as np
+from sample.utils import dsp as dsp_utils
 from scipy import fft
 
 
@@ -31,10 +32,11 @@ def dft(
   x_z[-hw:] = xw[:hw]
 
   x_fft = fft.rfft(x_z)
-  ax = np.maximum(np.abs(x_fft), np.finfo(float).eps)  # avoid zeros in log
-  mx = 20 * np.log10(ax)
+  ax = np.abs(x_fft)
+  ax = np.maximum(ax, np.finfo(float).eps, out=ax)  # avoid zeros in log
+  mx = dsp_utils.a2db.floorless(ax)
 
-  x_fft[ax < tol] = complex(0)
+  x_fft[ax < tol] = complex(0)  # pylint: disable=E1137
   px = np.unwrap(np.angle(x_fft))
   return mx, px
 
@@ -58,14 +60,18 @@ def peak_detect(x: np.ndarray, t: Optional[float] = None) -> np.ndarray:
       np.logical_and, conditions)) + 1  # compensate for skipping first sample
 
 
-def peak_refine(mx: np.ndarray, px: np.ndarray,
-                ploc: np.ndarray) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+def peak_refine(
+    ploc: np.ndarray,
+    mx: np.ndarray,
+    px: Optional[np.ndarray] = None
+) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[np.ndarray, np.ndarray,
+                                                np.ndarray]]:
   """Refine detected peaks with parabolic approximation
 
   Arguments:
+    ploc (array): Peak locations
     mx (array): Magnitude spectrum in dB
     px (array): Phase spectrum
-    ploc (array): Peak locations
 
   Returns:
     (array, array, array): Interpolated peak locations, magnitudes and phases"""
@@ -77,6 +83,8 @@ def peak_refine(mx: np.ndarray, px: np.ndarray,
   ploc_d = .5 * dmx / (mx_l - 2 * mx_c + mx_r)
   ploc_i = ploc + ploc_d  # x-coordinate of vertex
   pmag_i = mx_c - .25 * dmx * ploc_d  # y-coordinate of vertex
+  if px is None:
+    return ploc_i, pmag_i
   pph_i = np.interp(  # linear interpolation for phase
       ploc_i, np.arange(0, px.size), px)
   return ploc_i, pmag_i, pph_i
@@ -84,8 +92,10 @@ def peak_refine(mx: np.ndarray, px: np.ndarray,
 
 def peak_detect_interp(
     mx: np.ndarray,
-    px: np.ndarray,
-    t: Optional[float] = None) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+    px: Optional[np.ndarray] = None,
+    t: Optional[float] = None
+) -> Union[Tuple[np.ndarray, np.ndarray], Tuple[np.ndarray, np.ndarray,
+                                                np.ndarray]]:
   """Detect peaks (local maxima) in a signal, refining the value with
   parabolic interpolation
 
@@ -96,4 +106,4 @@ def peak_detect_interp(
 
   Returns:
     (array, array, array): Interpolated peak locations, magnitudes and phases"""
-  return peak_refine(mx, px, peak_detect(mx, t=t))
+  return peak_refine(peak_detect(mx, t=t), mx, px)
